@@ -2,6 +2,7 @@ from . import io, log
 from .aggregate import aggregate, aggregate_async
 from .backoff import asyncbackoff, asyncretry
 from .circuit_breaker import CircuitBreaker, CircuitBroken, cutout
+from .compat import context_partial
 from .context import Context, get_context
 from .counters import Statistic, get_statistics
 from .entrypoint import entrypoint, run
@@ -13,9 +14,9 @@ from .process_pool import ProcessPoolExecutor
 from .service import Service
 from .signal import Signal, receiver
 from .thread_pool import (
-    IteratorWrapperSeparate, ThreadPoolExecutor, context_partial,
+    IteratorWrapperSeparate, ThreadPoolExecutor, sync_await,
     sync_wait_coroutine, threaded, threaded_iterable,
-    threaded_iterable_separate, threaded_separate,
+    threaded_iterable_separate, threaded_separate, wait_coroutine,
 )
 from .timeout import timeout
 from .utils import (
@@ -61,10 +62,12 @@ __all__ = (
     "run",
     "select",
     "shield",
+    "sync_await",
     "sync_wait_coroutine",
     "threaded",
     "threaded_iterable",
     "threaded_iterable_separate",
     "threaded_separate",
     "timeout",
+    "wait_coroutine",
 )
