@@ -4,6 +4,7 @@ from abc import abstractmethod
 from functools import partial
 from typing import Any, Awaitable, Callable, Optional, Union
 
+from ..compat import get_running_loop
 from ..utils import OptionsType, awaitable, bind_socket
 from .base import SimpleServer
 
@@ -29,7 +30,7 @@ class UDPServer(SimpleServer):
 
         def connection_made(self, transport: Any) -> None:
             self.transport = transport
-            self.loop = asyncio.get_event_loop()
+            self.loop = get_running_loop()
 
         def datagram_received(self, data: bytes, addr: tuple) -> None:
             self.task_factory(self.handler(data, addr))
