@@ -7,29 +7,20 @@ from functools import wraps
 from multiprocessing import cpu_count
 from random import getrandbits
 from typing import (
-    Any, Awaitable, Callable, Iterable, List, Optional, Tuple, TypeVar, Union,
+    Any, Awaitable, Callable, Collection, Coroutine, Generator, Iterable,
+    Iterator, List, Optional, Set, Tuple, TypeVar, Union,
 )
 
+from .compat import (
+    event_loop_policy, get_running_loop, sock_set_nodelay, sock_set_reuseport,
+    time_ns,
+)
 from .thread_pool import ThreadPoolExecutor
 
 
 T = TypeVar("T")
 TimeoutType = Union[int, float]
 
-try:
-    from time import time_ns
-except ImportError:
-    from time import time
-
-    def time_ns() -> int:
-        return int(time() * 1000000000)
-
-try:
-    import uvloop  # type: ignore
-    event_loop_policy = uvloop.EventLoopPolicy()
-except ImportError:
-    event_loop_policy = asyncio.DefaultEventLoopPolicy()
-
 
 log = logging.getLogger(__name__)
 
@@ -66,30 +57,9 @@ def chunk_list(iterable: Iterable[T], size: int) -> Iterable[List[T]]:
 OptionsType = Iterable[Tuple[int, int, int]]
 
 
-if hasattr(socket, "TCP_NODELAY"):
-    def sock_set_nodelay(sock: socket.socket) -> None:
-        if sock.proto != socket.IPPROTO_TCP:
-            return None
-        sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
-else:
-    def sock_set_nodelay(sock: socket.socket) -> None:
-        return None
-
-if hasattr(socket, "SO_REUSEPORT"):
-    def sock_set_reuseport(sock: socket.socket, reuse_port: bool) -> None:
-        sock.setsockopt(
-            socket.SOL_SOCKET, socket.SO_REUSEPORT, int(reuse_port),
-        )
-else:
-    def sock_set_reuseport(sock: socket.socket, reuse_port: bool) -> None:
-        log.debug(
-            "SO_REUSEPORT is not implemented by underlying library. Skipping.",
-        )
-
-
 def bind_socket(
     *args: Any, address: str, port: int = 0, options: OptionsType = (),
-    reuse_addr: bool = True, reuse_port: bool = False,
+    reuse_addr: bool = True, reuse_port: bool = True,
     proto_name: Optional[str] = None
 ) -> socket.socket:
     """
@@ -120,9 +90,12 @@ def bind_socket(
     if not args and ":" in address:
         sock.setsockopt(socket.IPPROTO_IPV6, socket.IPV6_V6ONLY, 0)
 
-    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, reuse_addr.real)
+    if reuse_addr:
+        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+
+    if reuse_port:
+        sock_set_reuseport(sock, True)
 
-    sock_set_reuseport(sock, reuse_port)
     sock_set_nodelay(sock)
 
     for level, option, value in options:
@@ -198,7 +171,9 @@ def new_event_loop(
     return loop
 
 
-def shield(func: Callable[..., Awaitable[T]]) -> Callable[..., Awaitable[T]]:
+def shield(
+    func: Callable[..., Coroutine[Any, Any, T]],
+) -> Callable[..., Coroutine[Any, Any, T]]:
     """
     Simple and useful decorator for wrap the coroutine to `asyncio.shield`.
 
@@ -212,19 +187,27 @@ def shield(func: Callable[..., Awaitable[T]]) -> Callable[..., Awaitable[T]]:
         return await future
 
     @wraps(func)
-    def wrap(*args: Any, **kwargs: Any) -> Awaitable[Any]:
+    def wrap(*args: Any, **kwargs: Any) -> Coroutine[Any, Any, T]:
         return wraps(func)(awaiter)(asyncio.shield(func(*args, **kwargs)))
 
     return wrap
 
 
-class SelectResult:
+class SelectResult(Collection):
+    __slots__ = ("length", "result_idx", "is_exception", "value")
+
     def __init__(self, length: int):
         self.length = length
         self.result_idx = None      # type: Optional[int]
         self.is_exception = None    # type: Optional[bool]
         self.value = None           # type: Any
 
+    def __len__(self) -> int:
+        return self.length
+
+    def __contains__(self, x: Any) -> bool:
+        return self.value is x
+
     def set_result(self, idx: int, value: Any, is_exception: bool) -> None:
         if self.result_idx is not None:
             return
@@ -241,12 +224,130 @@ class SelectResult:
     def done(self) -> bool:
         return self.result_idx is not None
 
-    def __iter__(self) -> Iterable[Optional[T]]:
+    def __iter__(self) -> Iterator[Optional[T]]:
         for i in range(self.length):
-            if i == self.result_idx:
-                yield self.value
-            else:
-                yield None
+            yield self.value if i == self.result_idx else None
+
+
+class SelectAwaitable:
+    """
+    Select one of passed awaitables
+    """
+
+    __slots__ = (
+        "_awaitables", "_return_exceptions", "_timeout", "_wait",
+        "__loop", "_cancel", "_result",
+    )
+
+    _result: SelectResult
+
+    def __init__(
+        self, *awaitables: Awaitable[T],
+        return_exceptions: bool = False, cancel: bool = True,
+        timeout: Optional[TimeoutType] = None,
+        wait: bool = True, loop: Optional[asyncio.AbstractEventLoop] = None
+    ):
+        """
+        :param awaitables: awaitable objects
+        :param return_exceptions: if True exception will not be raised
+                                  just returned as result
+        :param cancel: cancel unfinished coroutines (default True)
+        :param timeout: execution timeout
+        :param wait: when False and ``cancel=True``, unfinished coroutines will
+                     be cancelled in the background.
+        :param loop: event loop
+        """
+        self.__loop = loop
+        self._awaitables = awaitables
+        self._cancel = cancel
+        self._return_exceptions = return_exceptions
+        self._timeout = timeout
+        self._wait = wait
+
+    @property
+    def loop(self) -> asyncio.AbstractEventLoop:
+        if self.__loop is None:
+            self.__loop = get_running_loop()
+        return self.__loop
+
+    async def __waiter(self, idx: int, awaitable: Awaitable[T]) -> None:
+        try:
+            ret = await awaitable
+        except asyncio.CancelledError:
+            raise
+        except Exception as e:
+            return self._result.set_result(idx, e, is_exception=True)
+        self._result.set_result(idx, ret, is_exception=False)
+
+    async def __run(
+        self, coroutines: Iterable[asyncio.Future],
+    ) -> SelectResult:
+        try:
+            _, pending = await asyncio.wait(
+                coroutines,
+                timeout=self._timeout,
+                return_when=asyncio.FIRST_COMPLETED,
+            )
+
+            if self._cancel:
+                cancelling = cancel_tasks(pending)
+
+                if self._wait:
+                    await cancelling
+
+            if self._result.is_exception and not self._return_exceptions:
+                self._result.result()
+            return self._result
+        except TimeoutError as e:
+            self._result.set_result(0, e, is_exception=True)
+            raise
+        except asyncio.CancelledError:
+            await cancel_tasks(coroutines)
+            raise
+
+    def __await__(self) -> Generator[Any, None, SelectResult]:
+        self._result = SelectResult(len(self._awaitables))
+        coroutines = [
+            asyncio.ensure_future(self.__waiter(idx, coroutine))
+            for idx, coroutine in enumerate(self._awaitables)
+        ]
+        # Prevent double __await__ call
+        del self._awaitables
+        return self.__run(coroutines).__await__()
+
+
+select = SelectAwaitable
+
+
+def pending_futures(
+    futures: Iterable[asyncio.Future],
+) -> Iterator[asyncio.Future]:
+    # Copying collection to ignore it
+    # changes during iteration
+    for future in tuple(futures):
+        if future.done():
+            continue
+        yield future
+
+
+def set_exception(
+    futures: Iterable[asyncio.Future],
+    exc: Union[BaseException] = asyncio.CancelledError(),
+) -> Set[asyncio.Task]:
+    cancelled_tasks = set()
+
+    for future in pending_futures(futures):
+        if isinstance(future, asyncio.Task):
+            future.cancel()
+            cancelled_tasks.add(future)
+        elif isinstance(future, asyncio.Future):
+            future.set_exception(exc)
+        else:
+            log.warning(
+                "Skipping object %r because it's not a Task or Future",
+                future,
+            )
+    return cancelled_tasks
 
 
 def cancel_tasks(tasks: Iterable[asyncio.Future]) -> asyncio.Future:
@@ -262,24 +363,8 @@ def cancel_tasks(tasks: Iterable[asyncio.Future]) -> asyncio.Future:
     if not tasks:
         return future
 
-    cancelled_tasks = []
     exc = asyncio.CancelledError()
-
-    for task in tasks:
-        if task.done():
-            continue
-
-        if isinstance(task, asyncio.Task):
-            task.cancel()
-            cancelled_tasks.append(task)
-
-        elif isinstance(task, asyncio.Future):
-            task.set_exception(exc)
-
-        else:
-            log.warning(
-                "Skipping object %r because it's not a Task or Future", task,
-            )
+    cancelled_tasks = set_exception(tasks, exc)
 
     if not cancelled_tasks:
         return future
@@ -293,66 +378,6 @@ def cancel_tasks(tasks: Iterable[asyncio.Future]) -> asyncio.Future:
     return waiter
 
 
-async def _select_waiter(
-    idx: int, awaitable: Awaitable[T],
-    result: SelectResult,
-) -> None:
-    try:
-        ret = await awaitable
-    except asyncio.CancelledError:
-        raise
-    except Exception as e:
-        return result.set_result(idx, e, is_exception=True)
-
-    result.set_result(idx, ret, is_exception=False)
-
-
-async def select(
-    *awaitables: Awaitable[Any],
-    return_exceptions: bool = False, cancel: bool = True,
-    timeout: Optional[TimeoutType] = None,
-    wait: bool = True, loop: Optional[asyncio.AbstractEventLoop] = None
-) -> SelectResult:
-    """
-
-    :param awaitables: awaitable objects
-    :param return_exceptions: if True exception will not be raised
-                              just returned as result
-    :param cancel: cancel unfinished coroutines (default True)
-    :param timeout: execution timeout
-    :param wait: when False and ``cancel=True``, unfinished coroutines will
-                 be cancelled in the background.
-    :param loop: event loop
-    """
-
-    loop = loop or asyncio.get_event_loop()
-    result = SelectResult(len(awaitables))
-
-    coroutines = [
-        loop.create_task(_select_waiter(idx, coroutine, result))
-        for idx, coroutine in enumerate(awaitables)
-    ]
-
-    _, pending = await loop.create_task(
-        asyncio.wait(
-            coroutines,
-            timeout=timeout,
-            return_when=asyncio.FIRST_COMPLETED,
-        ),
-    )
-
-    if cancel:
-        cancelling = cancel_tasks(pending)
-
-        if wait:
-            await cancelling
-
-    if result.is_exception and not return_exceptions:
-        result.result()
-
-    return result
-
-
 def awaitable(
     func: Callable[..., Union[T, Awaitable[T]]],
 ) -> Callable[..., Awaitable[T]]:
@@ -382,7 +407,7 @@ def awaitable(
         if hasattr(result, "__await__"):
             return result       # type: ignore
         if asyncio.iscoroutine(result) or asyncio.isfuture(result):
-            return result       # type: ignore
+            return result
 
         return awaiter(result)  # type: ignore
 
